#include "available-stubs.h"

#include <errno.h>
#include "checker_api.h"

/* compiled from: . */
#ifdef HAVE___errno_location
/* From `/usr/include/errnos.h:31'.  */
int *chkr_stub___errno_location (void)
     __asm__ (CHKR_PREFIX ("__errno_location"));
int *
chkr_stub___errno_location (void)
{
  int *res;
  res = __errno_location ();
  return res;
}
#endif /* HAVE___errno_location */

/* compiled from: . */
#ifdef HAVE____errno
/* From `/usr/include/errnos.h:31'.  */
int *chkr_stub____errno (void)
     __asm__ (CHKR_PREFIX ("___errno"));
int *
chkr_stub____errno (void)
{
  return ___errno ();
}
#endif /* HAVE____errno */

