$format "#define %1	%2"
$hformat "#define %1	%n"
M_IE_ET		"*error interno*"
M_I_IEI_CC_ET	"(iei) error interno: instrucción desconocida."
M_U_NZA_NZ_ET	"(nza) direccionamiento de zona nula."
M_W_WOB_TE_ET	"(wro) escritura/modificación de byte(s) de lectura sola."
M_U_BVH_HE_ET	"(bvh) violación de límites del bloque en el montón."
M_R_RFB_HE_ET	"(rfb) lectura en un bloque libre."
M_W_WFB_HE_ET	"(wfb) exritura/modificación de un bloque libre."
M_R_RUH_HE_ET	"(ruh) lectura de byte(s) no inicializados en un bloque."
M_R_RUS_ST_ET	"(rus) lectura de byte(s) no inicializados en la pila."
M_R_RZS_ST_ET	"(rsz) lectura de una zona roja en la pila."
M_W_WZS_ST_ET	"(wzs) escritura de una zona roja en la pila."
M_M_WUS_ST_ET	"(wus) modificación de byte(s) no inicializados en la pila."
M_M_MZS_ST_ET	"(mzs) modificación de una zona rojz en la pila."
M_U_NMA_VD_ET	"(nma) no se está direccionando memoria."
M_U_BVM_MM_ET	"(bvm) violación de límites en memory mapeada."
M_U_APD_MM_ET	"(apd) Denegación de privilegios de acceso en memoria mapeada."
M_U_WRS_SH_ET	"(wrs) escritura en una memoria compartida de lectura sola."
M_E_ZNE_VD_ET	"(zne) zona no ejecutable."
M_A_SBV_SG_ET	"(sbv) violación de límites de segmento."
M_M_FBA_FR_ET	"(fba) llamada a free con dirección no obtenida por malloc."
M_M_FFB_FR_ET	"(ffb) free para un bloque ya devuelto."
M_M_FNL_FR_ET	"(fnl) llamada a free con un argumento nulo."
M_M_SBA_BR_ET	"(sba) llamada a sbrk con desplazamiento erróneo."
M_M_FBM_FR_ET	"(fbm) llamada a free antes de malloc."
M_M_OOM_MC_ET	"(oom) advertencia: memoria virtual exhausta."
M_M_MNA_AL_ET	"(mna) llamada a memalign con un alineamiento nulo."
M_M_MBA_AL_ET	"(mba) alineamiento no es potencia de 2."
M_I_IEB_MA_ET	"(ieb) error interno durante la comprobación."
M_I_BBS_MA_ET	"(bbs) valor erróneo de bytes_per_state. Valor por defecto."
M_I_BKC_MA_ET	"(bkc) llamada a brk/sbrk."
M_I_OOS_SM_ET	"(oos) agotada la memoria del sistema."
M_I_BRS_MA_ET	"(brs) tamaño erróneo de zona roja."
M_I_TMS_SG_ET	"(tms) demasiadas señales recibidas a la vez."
M_I_IES_SC_ET	"(ies) error interno: no puedo comprobar esta llamada al sistema."
M_I_BFD_SC_ET	"(bfd) uso de descriptor de fichero reservado por Checker."
M_I_EFD_SC_ET	"(efd) descriptor de fichero fuera de límites."
M_I_SFD_SC_ET	"(sfd) sistema devuelve descriptor de fichero reservado por Checker."
M_C_MES_CK_ET	"(mes) mensaje de depuración para el usuario."
M_I_REC_MA_ET	"(rec) uso recursivo de malloc."
M_I_SIG_SG_ET	"(sig) señal."
M_I_BOC_SG_ET	"(bda) opción errónea para Checker."
M_I_GAR_MA_ET	"(gar) resultados del detector de memoria inutilizable."
M_I_PRF_MA_ET	"(prf) resultados de perfilamiento."
M_M_BAL_AL_ET	"(bal) alineamiento erróneo."
M_C_FUN_LB_ET	"(fun) advertencia de función."
