static char *help_message =
"Your Checker-ed program  reads options from the environment variable\n"
"CHECKEROPTS.  The options are\n"
"-s, --silent          Do not print the welcome message\n"
"-q, --quiet           Same as --silent\n"
"-a, --abort           Abort on startup\n"
"-h, --help            Print this message\n"
"--disp-args           Disp the command line and the environment\n"
"-n, --nosymtab        Do not use symbol table\n"
"-o=file, --output=file  Redirect Checker's output to `file'\n"
"-i=file, --image=file Set the image file (Checker finds it for you)\n"
"-p, --profile         Display profile information\n"
"-d=xx, --disable=xxx  Disable an address or a range of addresses\n"
"-S, --stop            Stop just before main\n"
"-D=end, --detector=end  Do leak detection at the end of the program\n"
"-m=a, --malloc0=a     Set the behavior of malloc(0)\n"
"-v, --verbose         Verbose\n"
"-u=end, --inuse=end   Do inuse at the end\n"
"--Wsignal=sig         Emit a warning when `sig' is received\n"
"--Wno-signal=sig      Do not emit a warning when `sig' is received\n"
"--Wsbrk --Wno-sbrk    Emit a warning if sbrk() is called\n"
"--Wfree-null          Emit a warning for free(0)\n"
"-A, --Wmemalign       Emit a warning if the aligment arg isn't a power of 2\n"
"--aged-queue=n        Set the size of the aged block queue\n"
"--leak-size-threshold=n Minimum size of a leak to be displayed\n"
"--bytes-per-state=n   Number of bytes handled by a bitmap state\n"
"--no-signals          Disable the signal manager\n"
"-t, --trace           Trace calls to malloc, free...\n"
"-w, --weak-check-copy Only copy the bitmap for memcpy, memmove, bcopy\n"
"-N=s, --NULLzone=s    Set the NULL zone size to s bytes\n"
"--exit-status	       exit() with a failure if there was a warning\n"
;

