/* test of inline with Checker.  */
#include <stdio.h>

inline int my_abs (int val)
{
  return (val < 0) ? -val: val;
}


void sub1(void);

int
main (void)
{
  int i;

  for (i = -5; i < 5; i++)
    printf ("abs(%d) = %d\n", i, my_abs (i));
  sub1();
  return 0;
}

void test (int (*func)(int))
{
  int i;
  for (i = -5; i < 5; i++)
    printf ("2: abs(%d) = %d\n", i, (*func) (i));
}

void
sub1 (void)
{
  int (*func)(int a);
  int i;

  func = my_abs;
  test (func);
}

  
