$format "#define %1	%2"
$hformat "#define %1	%n"
M_IE_ET		"*erreur interne*"
M_I_IEI_CC_ET	"(iei) erreur interne: instruction inconnue."
M_U_NZA_NZ_ET	"(nza) zone NULL addressée."
M_W_WOB_TE_ET	"(wro) écriture/modification d'octet(s) en lecture seule."
M_U_BVH_HE_ET	"(bvh) violation des limites d'un block dans le tas."
M_R_RFB_HE_ET	"(rfb) lecture dans un block libre."
M_W_WFB_HE_ET	"(wfb) écriture/modification d'un bloc libre."
M_R_RUH_HE_ET	"(ruh) lecture d'octet(s) non initialisé(s) dans un bloc."
M_R_RUS_ST_ET	"(rus) lecture d'octet(s) non initialisé(s) dans la pile."
M_R_RZS_ST_ET	"(rsz) lecture sur une zone interdite de la pile."
M_W_WZS_ST_ET	"(wzs) écriture sur une zone interdite de la pile."
M_M_WUS_ST_ET	"(wus) modification d'octet(s) non initialisé(s) dans la pile."
M_M_MZS_ST_ET	"(mzs) modification dans une zone interdite de la pile."
M_U_NMA_VD_ET	"(nma) n'adresse pas dans la mémoire."
M_U_BVM_MM_ET	"(bvm) violation de limites dans la mémoire mappée."
M_U_APD_MM_ET	"(apd) accès interdit dans la mémoire mappée."
M_U_WRS_SH_ET	"(wrs) écriture dans une mémoire partagée en lecture seule."
M_E_ZNE_VD_ET	"(zne) zone non éxécutable."
M_A_SBV_SG_ET	"(sbv) violation des limites d'un segment."
M_M_FBA_FR_ET	"(fba) free appelé avec une mauvaise adresse."
M_M_FFB_FR_ET	"(ffb) libération d'un bloc déjà libéré."
M_M_FNL_FR_ET	"(fnl) free appelé avec un argument nul."
M_M_SBA_BR_ET	"(sba) sbrk appelé avec un mauvais offset."
M_M_FBM_FR_ET	"(fbm) free appelé avant malloc."
M_M_OOM_MC_ET	"(oom) attention: mémoire virtuelle dépassée."
M_M_MNA_AL_ET	"(mna) memalign appelé avec un alignement nul."
M_M_MBA_AL_ET	"(mba) l'alignement n'est pas une puissance de 2."
M_I_IEB_MA_ET	"(ieb) erreur interne lors d'une vérification."
M_I_BBS_MA_ET	"(bbs) mauvaise valeur de bytes_per_state."
M_I_BKC_MA_ET	"(bkc) brk/sbrk a déjà été appelé."
M_I_OOS_SM_ET	"(oos) plus de mémoire système."
M_I_BRS_MA_ET	"(brs) mauvaise taille de la zone rouge."
M_I_TMS_SG_ET	"(tms) trop de signaux attrapés en même temps."
M_I_IES_SC_ET	"(ies) erreur interne: ne peut vérifier cet appel système."
M_I_BFD_SC_ET	"(bfd) un fd réservé par Checker est utilisé."
M_I_EFD_SC_ET	"(efd) descripteur de fichier en dehors des limites."
M_I_SFD_SC_ET	"(sfd) OS retourne un fd réservé par Checker."
M_C_MES_CK_ET	"(mes) message pour l'utilisateur."
M_I_REC_LC_ET	"(rec) utilisation récursive de malloc."
M_I_SIG_SG_ET	"(sig) signal."
M_I_BOC_SG_ET	"(bda) mauvaise option pour for Checker."
M_I_GAR_MA_ET	"(gar) résultats du garbage detector."
M_I_PRF_MA_ET	"(prf) informations de profile."
M_M_BAL_AL_ET	"(bal) bad alignment."
M_C_FUN_LB_ET	"(fun) function warning."
