#if 1
#define __KERNEL__
#include <linux/ipc.h>
/* #ifndef LINUX_GLIBC1
#include <asm/ipc.h>
#endif */

struct shmid_ds;

extern int chkr__ipc (int call, int first, int second, int third, void *ptr);
int chkr_shmctl (int shmid, int cmd, struct shmid_ds *buf);
			
int
chkr_shmctl (int shmid, int cmd, struct shmid_ds *buf)
{
    return chkr__ipc (SHMCTL, shmid, cmd, 0, buf);
}
#else

#include <sys/ipc.h>
#include <sys/shm.h>

int
chkr_shmctl (int shmid, int cmd, struct shmid_ds *buf)
{
  return shmctl(shmid, cmd, buf);
}
#endif
