$header "static char *abort_message[]"
M_ABORT_M1		"L'option -a fait arrêter Checker ici. Bye.\n"

$header "static char* init_message[]"
$literal "#if !defined(SIMCHECKER) && !defined(GCCCHECKER)"
M_COPY_M1		"Ce programme a été compilé avec `checkergcc' ou `checkerg++'.\n"
$literal "#endif"
M_COPY_M2		"Checker est un détecteur d'accès à la mémoire.\n"
M_COPY_M3		"Checker est distribué dans l'espoir d'être utile,\n"
M_COPY_M4		"cependant SANS AUCUNE GARANTIE; même sans la garantie implicite de\n"
M_COPY_M5		"COMMERCIALISATION ou d'aptitude à un but particulier. Voir la GNU\n"
M_COPY_M6		"General Public License pour plus de détails.\n"
M_COPY_M7		"Pour plus d'informations, mettre CHECKEROPTS à `--help'\n"
$end

$literal "#ifdef NEED_CHKR_LD_OPTIONS"
$header "static char* link_with_other_message[]"
M_L_W_O_M1		"Attention: cet éxécutable contient des fichiers non compilés avec Checker.\n"
M_L_W_O_M2		" Checker va seulement tester les violations des limites dans le tas.\n"
M_L_W_O_M3		"\n"
$end
$literal "#endif"
