$format "#define %1	%2"
$hformat "#define %1	%n"
M_IE_ET		"*interner Fehler*"
M_I_IEI_CC_ET	"(iei) interner Fehler: unbekannter Befehl."
M_U_NZA_NZ_ET	"(nza) Zugriff auf Null Zone."
M_W_WOB_TE_ET	"(wro) Schreiben/verändern von read-only byte(s)."
M_U_BVH_HE_ET	"(bvh) Blockgrenzenübertritt im Haufen."
M_R_RFB_HE_ET	"(rfb) Lesen eines zurückgegebenen Blocks."
M_W_WFB_HE_ET	"(wfb) Schreiben/verändern eines zurückgegebenen Blocks."
M_R_RUH_HE_ET	"(ruh) Lesen von nicht initialisierten byte(s) in einem Block."
M_R_RUS_ST_ET	"(rus) Lesen von nicht initialisierten byte(s) auf dem Stapel."
M_R_RZS_ST_ET	"(rsz) Lesen einer roten Zone auf dem Stapel."
M_W_WZS_ST_ET	"(wzs) Schreiben in einer roten Zone auf dem Stapel."
M_M_WUS_ST_ET	"(wus) Verändern nicht initialisierter byte(s) auf dem Stapel."
M_M_MZS_ST_ET	"(mzs) Verändern einer roten Zone auf dem Stapel."
M_U_NMA_VD_ET	"(nma) Keine Speicheradresse."
M_U_BVM_MM_ET	"(bvm) Grenzenübertriff in mapped memory."
M_U_APD_MM_ET	"(apd) Zugriffsverweigerung in mapped memory."
M_U_WRS_SH_ET	"(wrs) Schreiben in read-only shared memory."
M_E_ZNE_VD_ET	"(zne) Zone ist nicht ausführbar."
M_A_SBV_SG_ET	"(sbv) Segmentgrenzenübertritt."
M_M_FBA_FR_ET	"(fba) Aufruf von free mit nicht von malloc erhalt. Adresse."
M_M_FFB_FR_ET	"(ffb) Rückgabe von zurückgegebenem Block."
M_M_FNL_FR_ET	"(fnl) Aufruf von free mit einem Null Argument."
M_M_SBA_BR_ET	"(sba) Aufruf von sbrk mit falschem Offset."
M_M_FBM_FR_ET	"(fbm) Aufruf von free vor malloc."
M_M_OOM_MC_ET	"(oom) Warnung: Virtueller Speicher erschöpft."
M_M_MNA_AL_ET	"(mna) Aufruf von memalign mit einem Null Argument."
M_M_MBA_AL_ET	"(mba) Ausrichtung ist keine Zweierpotenz."
M_I_IEB_MA_ET	"(ieb) Interner Fehler bei Überprüfung."
M_I_BBS_MA_ET	"(bbs) Schlechter bytes_per_state Wert. Verwende default."
M_I_BKC_MA_ET	"(bkc) Aufruf von brk/sbrk."
M_I_OOS_SM_ET	"(oos) Kein Systemspeicher zur Verfügung."
M_I_BRS_MA_ET	"(brs) Schlechte Größe der roten Zone."
M_I_TMS_SG_ET	"(tms) Zu viele Signale gleichzeitig empfangen."
M_I_IES_SC_ET	"(ies) Interner Fehler: Kann diesen Systemaufruf nicht prüfen."
M_I_BFD_SC_ET	"(bfd) Zugriff auf Checkers Dateideskriptor."
M_I_EFD_SC_ET	"(efd) Dateideskriptor außerhalb des gültigen Bereichs."
M_I_SFD_SC_ET	"(sfd) System gibt von Checker gebrauchten Dateideskritor."
M_C_MES_CK_ET	"(mes) Debugnachricht für den Benutzer."
M_I_REC_MA_ET	"(rec) Recursiver Gebrauch von malloc."
M_I_SIG_SG_ET	"(sig) Signal."
M_I_BOC_SG_ET	"(bda) Ungültige Checker Option."
M_I_GAR_MA_ET	"(gar) Ergebnis des Garbage Detektors."
M_I_PRF_MA_ET	"(prf) Profileergebnis."
M_M_BAL_AL_ET	"(bal) Schlechte Ausrichtung."
M_C_FUN_LB_ET	"(fun) Funktionswarnung."
