#include "available-stubs.h"
#include "checker_api.h"

extern int yywrap (void);

/* compiled from: . */
#ifdef HAVE_yywrap
/* From `lex.h:1'.  */
int chkr_stub_yywrap (void ) __asm__ (CHKR_PREFIX ("yywrap"));
int
chkr_stub_yywrap (void )
{
#if USE_BI_JUMP
  __builtin_jump (yywrap);
#else
  return yywrap ();
#endif /* !USE_BI_JUMP */
}
#endif /* HAVE_yywrap */

