	.file	"chkr-crti.S"
	.section	".init"
	ld	[%sp + 96 + 96], %o0	! argc
	add	%sp, 96 + 32 + 68, %o1	! argv
	! envp = argv + argc + 1
	sll	%o0, 2, %l0
	add	%l0, 4, %l0
	add	%o1, %l0, %o2		! envp
	call	startup
	nop

	