/ Reversed stubs for main.
/   Copyright 1997 Tristan Gingold
/		  Written December 1997 by Tristan Gingold
/
/ This program is free software; you can redistribute it and/or
/ modify it under the terms of the GNU General Public License as
/ published by the Free Software Foundation; either version 2 of the
/ License, or (at your option) any later version.
/
/ This library is distributed in the hope that it will be useful,
/ but WITHOUT ANY WARRANTY; without even the implied warranty of
/ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/ General Public License for more details.
/
/ You should have received a copy of the GNU General Public License 
/ along with this program; see the file COPYING.  If not, write to
/ the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
/ Boston, MA 02111-1307, USA.
/
/ The author may be reached by US/French mail:
/		Tristan Gingold 
/		8 rue Parmentier
/		F-91120 PALAISEAU
/		FRANCE
/

	.file	"stubs-main.S"
	.section	".text"
	.globl	main
main:	jmp	chkr.main


	
