#include "checker_api.h"

/* A reverse stub: main. 
   crt will call "main", which is defined here, and which calls programmer
   main.  */

int chkr_stub_main (int argc, char *argv[], char *envp[])
     __asm__ (CHKR_PREFIX ("main"));
int this_main (int argc, char *argv[], char *envp[]) __asm__ ("main");
int
this_main (int argc, char *argv[], char *envp[])
{
  return chkr_stub_main (argc, argv, envp);
}
