$header "static char *abort_message[]"
M_ABORT_M1		"Checker aborted by -a option."

$header "static char* init_message[]"
$literal "#ifdef GCCCHECKER"
M_COPY_M1		"This program has been compiled with `checkergcc' or `checkerg++'.\n"
$literal "#endif"
M_COPY_M2		"Checker is a memory access detector.\n"
M_COPY_M3		"Checker is distributed in the hope that it will be useful,\n"
M_COPY_M4		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
M_COPY_M5		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n"
M_COPY_M6		"General Public License for more details.\n"
M_COPY_M7		"For more information, set CHECKEROPTS to `--help'\n"
$end

