char *sysnames[] = {
"SYS_syscall",
"SYS_exit",
"SYS_fork",
"SYS_read",
"SYS_write",
"SYS_open",
"SYS_close",
"SYS_wait",
"SYS_creat",
"SYS_link",
"SYS_unlink",
"SYS_exec",
"SYS_chdir",
"SYS_time",
"SYS_mknod",
"SYS_chmod",
"SYS_chown",
"SYS_brk",
"SYS_stat",
"SYS_lseek",
"SYS_getpid",
"SYS_mount",
"SYS_umount",
"SYS_setuid",
"SYS_getuid",
"SYS_stime",
"SYS_ptrace",
"SYS_alarm",
"SYS_fstat",
"SYS_pause",
"SYS_utime",
"SYS_stty",
"SYS_gtty",
"SYS_access",
"SYS_nice",
"SYS_statfs",
"SYS_sync",
"SYS_kill",
"SYS_fstatfs",
"SYS_pgrpsys",
"SYS_xenix",
"SYS_dup",
"SYS_pipe",
"SYS_times",
"SYS_profil",
"SYS_plock",
"SYS_setgid",
"SYS_getgid",
"SYS_signal",
"SYS_msgsys",
"SYS_syssun",
"SYS_acct",
"SYS_shmsys",
"SYS_semsys",
"SYS_ioctl",
"SYS_uadmin",
"(unknown)",
"SYS_utssys",
"SYS_fdsync",
"SYS_execve",
"SYS_umask",
"SYS_chroot",
"SYS_fcntl",
"SYS_ulimit",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"SYS_rmdir",
"SYS_mkdir",
"SYS_getdents",
"(unknown)",
"(unknown)",
"SYS_sysfs",
"SYS_getmsg",
"SYS_putmsg",
"SYS_poll",
"SYS_lstat",
"SYS_symlink",
"SYS_readlink",
"SYS_setgroups",
"SYS_getgroups",
"SYS_fchmod",
"SYS_fchown",
"SYS_sigprocmask",
"SYS_sigsuspend",
"SYS_sigaltstack",
"SYS_sigaction",
"SYS_sigpending",
"SYS_context",
"SYS_evsys",
"SYS_evtrapret",
"SYS_statvfs",
"SYS_fstatvfs",
"(unknown)",
"SYS_nfssys",
"SYS_waitsys",
"SYS_sigsendsys",
"SYS_hrtsys",
"SYS_acancel",
"SYS_async",
"SYS_priocntlsys",
"SYS_pathconf",
"SYS_mincore",
"SYS_mmap",
"SYS_mprotect",
"SYS_munmap",
"SYS_fpathconf",
"SYS_vfork",
"SYS_fchdir",
"SYS_readv",
"SYS_writev",
"SYS_xstat",
"SYS_lxstat",
"SYS_fxstat",
"SYS_xmknod",
"SYS_clocal",
"SYS_setrlimit",
"SYS_getrlimit",
"SYS_lchown",
"SYS_memcntl",
"SYS_getpmsg",
"SYS_putpmsg",
"SYS_rename",
"SYS_uname",
"SYS_setegid",
"SYS_sysconfig",
"SYS_adjtime",
"SYS_systeminfo",
"(unknown)",
"SYS_seteuid",
"SYS_vtrace",
"SYS_fork1",
"SYS_sigtimedwait",
"SYS_lwp_info",
"SYS_yield",
"SYS_lwp_sema_wait",
"SYS_lwp_sema_post",
"(unknown)",
"(unknown)",
"(unknown)",
"SYS_modctl",
"SYS_fchroot",
"SYS_utimes",
"SYS_vhangup",
"SYS_gettimeofday",
"SYS_getitimer",
"SYS_setitimer",
"SYS_lwp_create",
"SYS_lwp_exit",
"SYS_lwp_suspend",
"SYS_lwp_continue",
"SYS_lwp_kill",
"SYS_lwp_self",
"SYS_lwp_setprivate",
"SYS_lwp_getprivate",
"SYS_lwp_wait",
"SYS_lwp_mutex_unlock",
"SYS_lwp_mutex_lock",
"SYS_lwp_cond_wait",
"SYS_lwp_cond_signal",
"SYS_lwp_cond_broadcast",
"SYS_pread",
"SYS_pwrite",
"SYS_llseek",
"SYS_inst_sync",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"(unknown)",
"SYS_acl",
"SYS_auditsys",
"SYS_processor_bind",
"SYS_processor_info",
"SYS_p_online",
"SYS_sigqueue",
"SYS_clock_gettime",
"SYS_clock_settime",
"SYS_clock_getres",
"SYS_timer_create",
"SYS_timer_delete",
"SYS_timer_settime",
"SYS_timer_gettime",
"SYS_timer_getoverrun",
"SYS_nanosleep",
"SYS_facl"
};
