#include <stdarg.h>

struct doscan_arg
{
int len;
const char *s2;
const char *s1;
char a;
char b;
};

int
vsscanf (const char *s, const char *format, va_list par)
{
  struct doscan_arg da;

  da.a = 3;
  da.b = 20;
  da.s1 = s;
  da.s2 = s;
  da.len = strlen (s);
  return __doscan_u (&da, format, par);
}
