/* Some systems dependant definitions for linux-gnulibc1
   Copyright 1998 Tristan Gingold
		  Written April 1998 by Tristan Gingold

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License 
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

The author may be reached by US/French mail:
		Tristan Gingold 
		8 rue Parmentier
		F-91120 PALAISEAU
		FRANCE
*/

/* Do you think it is bad style ?
   Perhaps, but this way allow to focus on the current version.  */
#include "config/i386/Linux/tm-linux.h"

#undef MMAP_ADDR
#define MMAP_ADDR __ptr_t

/* The second argument of sigaction is not const.  */
#define NO_CONST_FOR_SIGACTION_ARG_2
