/* Checker stubs for functions defined in libgcc.
   Copyright 1998 Tristan Gingold
		  Written August 1998 by Tristan Gingold

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License 
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

 The author may be reached by US/French mail:
		Tristan Gingold 
		8 rue Parmentier
		F-91120 PALAISEAU
		FRANCE
*/
#include "available-stubs.h"

#include <sys/types.h>
#include "checker_api.h"

#ifdef HAVE_tinfo
#include "c++/tinfo.cc"
#endif

#ifdef HAVE_tinfo2
#include "c++/tinfo2.cc"
#endif

#ifdef HAVE_exception
#pragma implementation "exception"
#include "c++/exception.cc"
#endif

#ifdef HAVE_new
#pragma implementation "new"
#include "c++/new.cc"
#endif

#ifdef HAVE_opnew
#define L_op_new
#include "c++/new1.cc"
#endif

#ifdef HAVE_opnewnt
#define L_op_newnt
#include "c++/new1.cc"
#endif

#ifdef HAVE_opvnew
#define L_op_vnew
#include "c++/new2.cc"
#endif

#ifdef HAVE_opvnewnt
#define L_op_vnewnt
#include "c++/new2.cc"
#endif

#ifdef HAVE_opdel
#define L_op_delete
#include "c++/new2.cc"
#endif

#ifdef HAVE_opdelnt
#define L_op_delnt
#include "c++/new2.cc"
#endif

#ifdef HAVE_opvdel
#define L_op_vdel
#include "c++/new2.cc"
#endif

#ifdef HAVE_opvdelnt
#define L_op_vdelnt
#include "c++/new2.cc"
#endif




