$literal '#define M_NO_MEM "nothingness segment"'

$header "static char *short_right_name[]"
M_2_NO_RIGHT		"--"
M_2_RO_RIGHT		" R"
M_2_WO_RIGHT		" W"
M_2_RW_RIGHT		"RW"

$header "static char *right_name[]"
M_IE_RIGHT_NAME		"*Interner Fehler*"
M_RO_RIGHT_NAME		"Beim Lesen"
M_WO_RIGHT_NAME		"Beim Schreiben"
M_RW_RIGHT_NAME		"Beim Verändern"
