$header "static char *abort_message[]"
M_ABORT_M1		"Checker abortado por opción -a."

$header "static char* help_message[]"
M_HELP_M1		"Checker lee opciones de la variable de entorno CHECKEROPTS\n"
M_HELP_M2		"Options son:\n"
M_HELP_M3		"-s --silent          Sin mensave de bienvenida.\n"
M_HELP_M4		"-q --quiet           Lo mismo que --silent.\n"
M_HELP_M5		"-a --abort           Abortar al arranque.\n"
M_HELP_M6		"-h --help            Imprimir este mensaje.\n"
$literal "#ifdef CHKR_SAVESTACK"
M_HELP_M7		"-n --nosymtab        No usar tabla de símbolos.\n"
$literal "#endif"
M_HELP_M8		"-o=file --output=file  Enviar salida de Checker a `file'.\n"
M_HELP_M9		"-i=file --image=file Define el fichero image (Checker lo encontrará para tí)\n"
$literal "#ifdef CHKR_PROFILE"
M_HELP_M10		"-p --profile         Ver información de perfilamiento.\n"
$literal "#endif"
M_HELP_M11		"-d=xx --disable=xxx  Deshabilitar una dirección o un rango de direcciones.\n"
M_HELP_M12		"-S --stop            Para justamente antes de main.\n"
M_HELP_M13		"-D=end --detector=end    Detección de agujeros al final del programa.\n"
M_HELP_M14		"-m=a  --malloc0=a    Define comportamiento de malloc(0).\n"
M_HELP_M15		"-v    --verbose      Más labioso.\n"
M_HELP_M16		"-u=end --inuse=end   Haz inuse al final.\n"
M_HELP_M17		" -Wsignal=sig        Advierte la recepción de `sig'.\n"
M_HELP_M18		" -Wno-signal=sig     No advertir la recepción de `sig'.\n"
M_HELP_M19		" --aged-queue=n      Define el tamaño de la cola de bloques con edad.\n"
M_HELP_M20		" --leak-size-threshold=n Tamaño mínimo de un agujero para mostrarlo.\n"
M_HELP_M21		" --bytes-per-state=n Número de bytes gestionados por un estado de mapa de bits.\n"
M_HELP_M22		" --no-signals        Deshabilitar el gestor de señales.\n"
M_HELP_M23		" --Wsbrk --Wno-sbrk  Advertir la llamada a sbrk().\n"
M_HELP_M24		"-A --Wmemalign       Advertir un argumento de alineamiento que no sea una potencia de 2.\n"
M_HELP_M25		"-t --trace           Trazar llamadas a malloc, free...\n"
M_HELP_M26		"-w --weak-check-copy Copiar mapa de bits sólo para memcpy, memmove, bcopy\n"
M_HELP_M0		"\n"

$header "static char* init_message[]"
$literal "#if !defined(SIMCHECKER) && !defined(GCCCHECKER)"
M_COPY_M1		"Este programa ha sido compilado con `checkergcc' o `checkerg++'.\n"
$literal "#endif"
M_COPY_M2		"Checker es un detector de acceso a memoria.\n"
M_COPY_M3		"Se distribuye Checker en la esperanza que sea útil,\n"
M_COPY_M4		"pero SIN GARANTÍA; incluso sin la garantía implícita de\n"
M_COPY_M5		"VENDIBILIDAD o APTITUD PARA UN PROPÓSITO PARTICULAR.  Más detalles en\n"
M_COPY_M6		"GNU General Public License.\n"
M_COPY_M7		"Para más información define CHECKEROPTS con `--help'\n"
$end

$literal "#ifdef NEED_CHKR_LD_OPTIONS"
$header "static char* link_with_other_message[]"
M_L_W_O_M1		"Advertencia: este programa contiene funciones no compiladas con .\n"
M_L_W_O_M2		" Por ello Checker comprobará sólo las violaciones de límites en el montón.\n"
M_L_W_O_M3		"\n"
$end
$literal "#endif"
