$format "#define %1	%2"
$hformat "#define %1	%n"
M_IE_ET		"*internal error*"
M_I_IEI_CC_ET	"(iei) internal error: instruction unknown."
M_U_NZA_NZ_ET	"(nza) null zone addressed."
M_W_WOB_TE_ET	"(wro) write/modify read-only byte(s)."
M_U_BVH_HE_ET	"(bvh) block bounds violation in the heap."
M_R_RFB_HE_ET	"(rfb) read in a free block."
M_W_WFB_HE_ET	"(wfb) write/modify a free block."
M_R_RUH_HE_ET	"(ruh) read uninitialized byte(s) in a block."
M_R_RUS_ST_ET	"(rus) read uninitialized byte(s) in the stack."
M_R_RZS_ST_ET	"(rsz) read on a red zone in the stack."
M_W_WZS_ST_ET	"(wzs) write on a red zone in the stack."
M_M_WUS_ST_ET	"(wus) modify uninitialized byte(s) in the stack."
M_M_MZS_ST_ET	"(mzs) modify a red zone in the stack."
M_U_NMA_VD_ET	"(nma) no memory addressed."
M_U_BVM_MM_ET	"(bvm) bounds violation in mapped memory."
M_U_APD_MM_ET	"(apd) access permission denied in mapped memory."
M_U_WRS_SH_ET	"(wrs) write in a read-only shared memory."
M_E_ZNE_VD_ET	"(zne) zone not executable."
M_A_SBV_SG_ET	"(sbv) segment bound violation."
M_M_FBA_FR_ET	"(fba) free called with an address not obtained from malloc."
M_M_FFB_FR_ET	"(ffb) free an already free block."
M_M_FNL_FR_ET	"(fnl) free called with a null argument."
M_M_SBA_BR_ET	"(sba) sbrk called with a bad offset."
M_M_FBM_FR_ET	"(fbm) free called before malloc."
M_M_OOM_MC_ET	"(oom) warning: virtual memory exhausted."
M_M_MNA_AL_ET	"(mna) memalign called with a null alignment."
M_M_MBA_AL_ET	"(mba) alignment is not a power of 2."
M_I_IEB_MA_ET	"(ieb) internal error while checking."
M_I_BBS_MA_ET	"(bbs) bad bytes_per_state value. Used default value."
M_I_BKC_MA_ET	"(bkc) brk/sbrk has been called."
M_I_OOS_SM_ET	"(oos) out of system memory."
M_I_BRS_MA_ET	"(brs) bad red zone size."
M_I_TMS_SG_ET	"(tms) too many signals caught at the same time."
M_I_IES_SC_ET	"(ies) internal error: can't check this syscall."
M_I_BFD_SC_ET	"(bfd) file descriptor reserved by Checker is used."
M_I_EFD_SC_ET	"(efd) file descriptor out of range."
M_I_SFD_SC_ET	"(sfd) the system has returned a file descriptor reserved by Checker."
M_C_MES_CK_ET	"(mes) debugging message for the user."
M_I_REC_MA_ET	"(rec) recursive use of malloc."
M_I_SIG_SG_ET	"(sig) signal."
M_I_BOC_SG_ET	"(bda) bad option for Checker."
M_I_GAR_MA_ET	"(gar) garbage detector results."
M_I_PRF_MA_ET	"(prf) profile results."
M_M_BAL_AL_ET	"(bal) bad alignment."
M_C_FUN_LB_ET	"(fun) function warning."
