static char *help_message =
"Vous pouvez donner des options à Checker par la variable"
"d'environnement CHECKEROPTS.  Les options sont:\n"
"-s, --silent          N'affiche pas le message de début\n"
"-q, --quiet           idem\n"
"-a, --abort           Arrêt immédiat\n"
"-h, --help            Affiche ce message\n"
"--disp-args           Affiche la ligne de commande et l'environement\n"
"-n, --nosymtab        N'utilise pas la table des symbols.\n"
"-o=file, --output=file  Redirige les message de Checker dans `file'.\n"
"-i=file, --image=file Spécifie le fichier image\n"
"-p, --profile         Affiche des informations de profile.\n"
"-d=xx, --disable=xxx  Supprime les message pour l'adresse.\n"
"-S, --stop            Arrêt avant main().\n"
"-D=end, --detector=end Affiche les trous de memoire lors d'exit(3)\n"
"-m=a, --malloc0=a     Change le comportement de malloc(0)\n"
"-v, --verbose         Verbose\n"
"-u=end, --inuse=end   Affiche les blocs non libérés à la fin\n"
"--Wsignal=sig         Message quand `sig' est reçu\n"
"--Wno-signal=sig      Pas de message quand `sig' est reçu\n"
"--Wsbrk --Wno-sbrk    Emet un message si sbrk() est appelé\n"
"--Wfree-null          Emet un message si free(0) est utilise\n"
"-A, --Wmemalign       Emet un message si l'appel n'est pas conventionel\n"
"--aged-queue=n        Taille de la queue des blocs agés\n"
"--leak-size-threshold=n Taille minimum d'un trou pour être affiché\n"
"--bytes-per-state=n   Nombre d'octets par élément de bitmap\n"
"--no-signals          Supprime le gestionnaire de signaux\n"
"-t, --trace           Trace les appels à malloc, free...\n"
"-w, --weak-check-copy Copie les bitmaps pour memcpy, bcopy, memmove\n"
"-N=s, --NULLzone=s    Specifie la taille de la zone des NULL\n"
;
