/* Libraries that can be added, due to local configuration.  */
#include "config.h"

#ifdef HAVE_CURSES_H
library curses
#endif

#ifdef HAVE_TERMCAP_H
library termcap
#endif

#ifdef HAVE_NCURSES_H
library ncurses
#endif

#ifdef HAVE_GXX
library gcc2
#endif
