$header "static char *abort_message[]"
M_ABORT_M1		"Checker bricht ab wegen der -a Option."

$header "static char* help_message[]"
M_HELP_M1		"Checker liest Optionen aus Umgebungsvariable CHECKEROPTS\n"
M_HELP_M2		"Optionen sind:\n"
M_HELP_M3		"-s --silent          Kein Willkommensgruß.\n"
M_HELP_M4		"-q --quiet           Wie --silent.\n"
M_HELP_M5		"-a --abort           Abbruch nach Start.\n"
M_HELP_M6		"-h --help            Dieser Text.\n"
$literal "#ifdef CHKR_SAVESTACK"
M_HELP_M7		"-n --nosymtab        Keine Symboltabelle verwenden.\n"
$literal "#endif"
M_HELP_M8		"-o=file --output=file  Sende Checkers Ausgabe in Datei `file'.\n"
M_HELP_M9		"-i=file --image=file Setze die image Datei (Checker sucht sie für dich)\n"
$literal "#ifdef CHKR_PROFILE"
M_HELP_M10		"-p --profile         Zeige Profile Information.\n"
$literal "#endif"
M_HELP_M11		"-d=xx --disable=xxx  Adresse oder Adreßrang abschalten.\n"
M_HELP_M12		"-S --stop            Halt gerade vor main.\n"
M_HELP_M13		"-D=end --detector=end    Nach Program Löchersuchen.\n"
M_HELP_M14		"-m=a  --malloc0=a    Verhalten von malloc(0).\n"
M_HELP_M15		"-v    --verbose      Geschwätziger.\n"
M_HELP_M16		"-u=end --inuse=end   Em Ende inuse.\n"
M_HELP_M17		" -Wsignal=sig        Warnung bei Empfang von Signal `sig'.\n"
M_HELP_M18		" -Wno-signal=sig     Keine Warnung bei Empfang von Signal `sig'.\n"
M_HELP_M19		" --aged-queue=n      Größe der Liste von Blöcken mit Alter.\n"
M_HELP_M20		" --leak-size-threshold=n Mindest Lochgröße zum Anzeigen.\n"
M_HELP_M21		" --bytes-per-state=n Anzahl von bytes durch bitmap Status behandelt.\n"
M_HELP_M22		" --no-signals        Signalhändler abschalten.\n"
M_HELP_M23		" --Wsbrk --Wno-sbrk  Warnung bei Aufruf von sbrk().\n"
M_HELP_M24		"-A --Wmemalign       Warnung wenn Ausrichtungsargument keine Zweierpotenz.\n"
M_HELP_M25		"-t --trace           Verfolge Aufrufe von malloc, free...\n"
M_HELP_M26		"-w --weak-check-copy Kopiere bitmap nur für memcpy, memmove, bcopy\n"
M_HELP_M0		"\n"

$header "static char* init_message[]"
$literal "#if !defined(SIMCHECKER) && !defined(GCCCHECKER)"
M_COPY_M1		"Diese Programm wurde mit`checkergcc' oder `checkerg++' compiliert.\n"
$literal "#endif"
M_COPY_M2		"Checker ist ein Speicherzugriffdetektor.\n"
M_COPY_M3		"Checker vertrieben in der Hoffnung, daß es nützlich sei,\n"
M_COPY_M4		"aber OHNE IRGENDEINE GARANTIE; auch ohne die implizite Garantie\n"
M_COPY_M5		"von VERKÄUFLICHKEIT oder EIGNUNG FÜR EINEN KONKRETEN ZWECK.  Siehe die GNU\n"
M_COPY_M6		"General Public License für Genaueres.\n"
M_COPY_M7		"Für mehr Information, setzen Sie CHECKEROPTS auf `--help'\n"
$end

$literal "#ifdef NEED_CHKR_LD_OPTIONS"
$header "static char* link_with_other_message[]"
M_L_W_O_M1		"Achtung: diese Programm enthält Funktionen, die nicht mit Checker kompiliert wurden.\n"
M_L_W_O_M2		" Daher prüft Checker nur die Grenzüberschreitungen des Haufens.\n"
M_L_W_O_M3		"\n"
$end
$literal "#endif"
