#define HAVE_ARPA_INET_H 1
#define HAVE_CTYPE_H 1
#define HAVE_NCURSES_H 1
#define HAVE_STDARG_H 1
#define HAVE_DIRENT_H 1
#define HAVE_FCNTL_H 1
#define HAVE_FNMATCH_H 1
#define HAVE_SYS_TYPES_H 1
#define HAVE_GRP_H 1
#define HAVE_SYS_IOCTL_H 1
#define HAVE_STDDEF_H 1
#define HAVE_MATH_H 1
#define HAVE_NETDB_H 1
#define HAVE_NETINET_IN_H 1
#define HAVE_PWD_H 1
#define HAVE_SETJMP_H 1
#define HAVE_SIGNAL_H 1
#define HAVE_STRING_H 1
#define HAVE_STDIO_H 1
#define HAVE_STDLIB_H 1
#define HAVE_SYS_RESOURCE_H 1
#define HAVE_SYS_SOCKET_H 1
#define HAVE_SYS_STAT_H 1
#define HAVE_SYS_TIME_H 1
#define HAVE_SYS_TIMEB_H 1
#define HAVE_SYS_UIO_H 1
#define HAVE_SYS_UTSNAME_H 1
#define HAVE_SYS_WAIT_H 1
#define HAVE_TERMIOS_H 1
#define HAVE_TIME_H 1
#define HAVE_UNISTD_H 1
#define HAVE_SYS_PARAM_H 1
#define HAVE_UTIME_H 1
#define HAVE_TERMCAP_H 1
#define HAVE_SYS_MMAN_H 1
#define HAVE_SYS_MNTTAB_H 1
#define HAVE_ASSERT_H 1
#define HAVE_RPC_RPCENT_H 1
#define HAVE_SIGINFO_H 1
#define HAVE_SYS_TIMES_H 1
#define HAVE_LOCALE_H 1
#define HAVE_CRYPT_H 1

/* doscan is provided.  */
#define HAVE_DOSCAN 1

/* sync(2) does not return any value.  */
#define FUNC_SYNC_VOID 1

/* llseek type is offset_t.  */
#define LLSEEK_OFFSET_T offset_t

/* Needed for XXX_r functions.  */
#define _REENTRANT
