#include "available-stubs.h"

#ifndef HAVE_RX_H
#include <rx.h>
#include "checker_api.h"

/* compiled from: . */
#ifdef HAVE_re_comp
char * chkr_stub_re_comp (const void *src)
  __asm__ (CHKR_PREFIX ("re_comp"));
char *
chkr_stub_re_comp (const void *src)
{
  stubs_chkr_check_addr (src, sizeof (char), CHKR_RO, "src");
  return re_comp (src);
}
#endif /* HAVE_re_comp */

#ifdef HAVE_re_exec
int chkr_stub_re_exec (const void *src)
  __asm__ (CHKR_PREFIX ("re_exec"));
int
chkr_stub_re_exec (const void *src)
{
  stubs_chkr_check_addr (src, sizeof (char), CHKR_RO, "src");
  return re_exec (src);
}
#endif /* HAVE_re_exec */

#endif /* HAVE_RX_H */
