#define HAVE_fopen
#define HAVE_freopen
#define HAVE_fflush
#define HAVE_fclose
#define HAVE_remove
#define HAVE_rename
#define HAVE_tmpfile
#define HAVE_tmpnam
#define HAVE_setvbuf
#define HAVE_setbuf
#define HAVE_fprintf
#define HAVE_printf
#define HAVE_sprintf
#define HAVE_vprintf
#define HAVE_vfprintf
#define HAVE_vsprintf
#define HAVE_fscanf
#define HAVE_scanf
#define HAVE_sscanf
#define HAVE_fgetc
#define HAVE_fgets
#define HAVE_fputc
#define HAVE_fputs
#define HAVE_getc
#define HAVE_getchar
#define HAVE_gets
#define HAVE_putc
#define HAVE_putchar
#define HAVE_puts
#define HAVE_ungetc
#define HAVE_fread
#define HAVE_fwrite
#define HAVE_fseek
#define HAVE_ftell
#define HAVE_rewind
#define HAVE_clearerr
#define HAVE_feof
#define HAVE_ferror
#define HAVE_perror

#define HAVE_isalnum
#define HAVE_isalpha
#define HAVE_iscntrl
#define HAVE_isdigit
#define HAVE_isgraph
#define HAVE_islower
#define HAVE_isprint
#define HAVE_isgraph
#define HAVE_islower
#define HAVE_isprint
#define HAVE_ispunct
#define HAVE_isspace
#define HAVE_isupper
#define HAVE_isxdigit

#define HAVE_strcpy
#define HAVE_strncpy
#define HAVE_strcat
#define HAVE_strncat
#define HAVE_strcmp
#define HAVE_strncmp
#define HAVE_strchr
#define HAVE_strrchr
#define HAVE_strspn
#define HAVE_strcspn
#define HAVE_strpbrk
#define HAVE_strstr
#define HAVE_strlen
#define HAVE_strerror
#define HAVE_strtok
#define HAVE_memcpy
#define HAVE_memmove
#define HAVE_memcmp
#define HAVE_memchr
#define HAVE_memset

#define HAVE_sin
#define HAVE_cos
#define HAVE_tan
#define HAVE_asin
#define HAVE_acos
#define HAVE_atan
#define HAVE_atan2
#define HAVE_sinh
#define HAVE_cosh
#define HAVE_tanh
#define HAVE_exp
#define HAVE_log
#define HAVE_log10
#define HAVE_pow
#define HAVE_sqrt
#define HAVE_ceil
#define HAVE_floor
#define HAVE_fabs
#define HAVE_ldesp
#define HAVE_frexp
#define HAVE_modf
#define HAVE_fmod

#define HAVE_atof
#define HAVE_atoi
#define HAVE_atol
#define HAVE_strtod
#define HAVE_strtol
#define HAVE_stroul
#define HAVE_rand
#define HAVE_srand
#define HAVE_calloc
#define HAVE_malloc
#define HAVE_realloc
#define HAVE_free
#define HAVE_abort
#define HAVE_exit
#define HAVE_atexit
#define HAVE_system
#define HAVE_getenv
#define HAVE_bsearch
#define HAVE_qsort
#define HAVE_abs
#define HAVE_labs
#define HAVE_div
#define HAVE_ldiv

#define HAVE_assert

#define HAVE_setjmp
#define HAVE_longjmp

#define HAVE_signal
#define HAVE_raise

#define HAVE_clock
#define HAVE_time
#define HAVE_difftime
#define HAVE_mktime
#define HAVE_asctime
#define HAVE_ctime
#define HAVE_gmtime
#define HAVE_localtime

