#include <stdio.h>

void constructor (void) __attribute__((constructor));

void
constructor (void)
{
  printf ("Hello world in constructor\n");
}

int
main (void)
{
  printf ("Hello world in main\n");
  return 0;
}
