#include "available-stubs.h"

#include <sys/ipc.h>
#include "checker_api.h"

/* compiled from: . */
#ifdef HAVE_ftok
/* From `/usr/include/sys/ipc.h:10'.  */
key_t chkr_stub_ftok (char *pathname, char proj)
     __asm__ (CHKR_PREFIX ("ftok"));
key_t
chkr_stub_ftok (char *pathname, char proj)
{
  stubs_chkr_check_addr (pathname, CHKR_RO, "pathname");
#if USE_BI_JUMP
  __builtin_jump (ftok);
#else
  return ftok (pathname, proj);
#endif /* !USE_BI_JUMP */
}
#endif /* HAVE_ftok */
