/* Test if putchar/putc macro are well-handled.
   With Checked, an #include <stdio.h> include a special file,
   which includes the real one and #undef standard macros.
   This is necessary, because these macros handle structure components that
   are not updated.

   It also test a reference to a global variable, stdout.  */

#include <stdio.h>

int
main(void)
{
  putchar ('a');
  putc ('z', stdout);
  return 0;
}
