/* Just check div, which returns a structure.  */
#include <stdio.h>
#include <stdlib.h>

int
main(int argc, char **argv)
{
  int a = 5;
  int b = 9;
  div_t res;

  res = div (b, a);
  printf ("quot: %d, rem: %d\n", res.quot, res.rem);
  return 0;
}
