#ifndef _CHKR_DIR_H_
#define _CHKR_DIR_H_

#include <stddef.h>
#include <dirent.h>

typedef struct 
{
  size_t dd_max;
  int dd_fd;
  struct dirent *dd_buf;
  int dd_loc;
  int dd_size;
} chkr_DIR;

#endif /* _CHKR_DIR_H_ */
